@@ -306,6 +306,7 @@ class Q2Db:
         # schema_columns["insert_session_id"] = {"datatype": "int"}
         # schema_columns["update_session_id"] = {"datatype": "int"}
         # schema_columns["user_lock"] = {"datatype": "char", "datalen": 1}
+        pass
 
     def create_column(self, column_definition):
         """migrate given 'column_definition' to database"""
@@ -385,9 +386,9 @@ class Q2Db:
             self.create_index(column_definition)
 
             log_column_definition = dict(column_definition)
-            log_column_definition["pk"] = None
-            log_column_definition["ai"] = None
-            log_column_definition["uk"] = None
+            log_column_definition["pk"] = ""
+            log_column_definition["ai"] = ""
+            log_column_definition["uk"] = ""
             log_column_definition["table"] = "log_" + log_column_definition["table"]
             self.create_column(log_column_definition)
         return True
@@ -408,9 +409,8 @@ class Q2Db:
     def run_migrate_sql(self, sql_cmd):
         self._cursor(sql_cmd)
         # if 'Group' in sql_cmd:
-        #     print(sql_cmd)
         if self.last_sql_error != "":
-            self.migrate_error_list.append(self.last_sql_error)
+            self.migrate_error_list.append(f"{self.last_sql_error}: {sql_cmd}")
             return False
         return True
 
@@ -451,11 +451,9 @@ class Q2Db:
 
     def commit(self):
         self._cursor("commit")
-        # self.db_cursor_class.commit()
 
     def rollback(self):
         self._cursor("rollback")
-        # self.db_cursor_class.rollback()
 
     def raw_insert(self, table_name="", record={}):
         """insert dictionary into table"""
@@ -483,7 +481,9 @@ class Q2Db:
             return True
 
     def insert(self, table_name="", record={}):
-        """insert dictionary into table"""
+        """
+        insert dictionary into table
+        """
         if not (table_name and record):
             return None
 
@@ -518,38 +518,38 @@ class Q2Db:
         primary_key_columns = self.get_primary_key_columns(table_name)
 
         aipk = ""
-        if len(primary_key_columns) == 1:
+        if not table_name.upper().startswith("LOG_") and len(primary_key_columns) == 1:
             for d in primary_key_columns:
                 if primary_key_columns[d].get("ai"):
                     aipk = d
 
         columns_list = [x for x in record if x in table_columns]
-        if not aipk:
-            # create primary key value
-            for x in primary_key_columns:
-                if x not in columns_list:
-                    columns_list.append(x)
-                is_string_data = True if ("char" in primary_key_columns[x]["datatype"]) else False
-                if is_string_data:
-                    primary_key_value = record.get(x, "")
-                else:
-                    primary_key_value = int_(record.get(x, 0))
-                while (
-                    self.cursor(
-                        sql=f"""select {self.escape_char}{x}{self.escape_char}
-                                from {table_name}
-                                where {self.escape_char}{x}{self.escape_char}='{primary_key_value}'
-                            """
-                    ).row_count()
-                    > 0
-                ):
+        if not table_name.upper().startswith("LOG_"):
+            if not aipk:
+                # create primary key value
+                for x in primary_key_columns:
+                    if x not in columns_list:
+                        columns_list.append(x)
+                    is_string_data = True if ("char" in primary_key_columns[x]["datatype"]) else False
                     if is_string_data:
-                        primary_key_value += "_"
+                        primary_key_value = record.get(x, "")
                     else:
-                        primary_key_value += 1
-                record[x] = primary_key_value
-        else:  # autoincrement
-            if not table_name.upper().startswith("LOG_"):
+                        primary_key_value = int_(record.get(x, 0))
+                    while (
+                        self.cursor(
+                            sql=f"""select {self.escape_char}{x}{self.escape_char}
+                                    from {table_name}
+                                    where {self.escape_char}{x}{self.escape_char}='{primary_key_value}'
+                                """
+                        ).row_count()
+                        > 0
+                    ):
+                        if is_string_data:
+                            primary_key_value += "."
+                        else:
+                            primary_key_value += 1
+                    record[x] = primary_key_value
+            else:  # autoincrement
                 for pkname in primary_key_columns:
                     if pkname in record:
                         del record[pkname]
@@ -634,7 +634,7 @@ class Q2Db:
                 return False
             else:
                 if not table_name.upper().startswith("LOG_") and table_name.upper() != "PLATFORM":
-                    self.insert("log_" + table_name, record)
+                    self.insert("log_" + table_name, dict(record))
                 return True
         else:
             self.last_sql_error = "Update requires a primary key column!"
@@ -709,6 +709,18 @@ class Q2Db:
                     return row[0]["ret"]
         return {}
 
+    def get_uniq_value(self, table_name, column, start_value):
+        datatype = self.db_schema.get_schema_attr(table_name, column)["datatype"]
+        if "int" in datatype or "dec" in datatype or "num" in datatype:
+            return self.cursor(f"""select min({column}) +1 as pkvalue
+                            from {table_name}
+                            where {column} >= {start_value}
+                                and {column}+1 not in
+                                    (select {column} from {table_name})
+                        """).r.pkvalue
+        else:
+            return start_value + "."
+
     def _dict_factory(self, cursor, row, sql):
         return {
             col[0]: f"{row[idx] if row[idx] is not None else ''}".rstrip()
